@@ -10,18 +10,20 @@ our @ISA = (qw(Exporter));
 # possible tags for export
 our %EXPORT_TAGS = (
 	Functions => [qw(raw2hex raw2base32 raw2base64)],
-	Constants => [qw(RHASH_CRC32 RHASH_CRC32C RHASH_MD4 RHASH_MD5 RHASH_SHA1
-		RHASH_TIGER RHASH_TTH RHASH_BTIH RHASH_ED2K RHASH_AICH RHASH_WHIRLPOOL
-		RHASH_RIPEMD160 RHASH_GOST RHASH_GOST_CRYPTOPRO RHASH_HAS160
-		RHASH_SNEFRU128 RHASH_SNEFRU256 RHASH_SHA224 RHASH_SHA256
-		RHASH_SHA384 RHASH_SHA512 RHASH_EDONR256 RHASH_EDONR512
-		RHASH_SHA3_224 RHASH_SHA3_256 RHASH_SHA3_384 RHASH_SHA3_512 RHASH_ALL)]
+	Constants => [qw(RHASH_CRC32 RHASH_CRC32C RHASH_MD4 RHASH_MD5
+		RHASH_SHA1 RHASH_TIGER RHASH_TTH RHASH_BTIH RHASH_ED2K
+		RHASH_AICH RHASH_WHIRLPOOL RHASH_RIPEMD160 RHASH_GOST94
+		RHASH_GOST94_CRYPTOPRO RHASH_GOST12_256 RHASH_GOST12_512
+		RHASH_SHA224 RHASH_SHA256 RHASH_SHA384 RHASH_SHA512
+		RHASH_SHA3_224 RHASH_SHA3_256 RHASH_SHA3_384 RHASH_SHA3_512
+		RHASH_HAS160 RHASH_EDONR256 RHASH_EDONR512
+		RHASH_SNEFRU128 RHASH_SNEFRU256 RHASH_ALL)]
 );
 
 Exporter::export_tags( );
 Exporter::export_ok_tags( qw(Functions Constants) );
 
-our $VERSION = '0.96';
+our $VERSION = '0.98';
 
 require XSLoader;
 XSLoader::load('Crypt::Rhash', $VERSION);
@@ -39,11 +41,11 @@ use constant RHASH_ED2K  => 0x80;
 use constant RHASH_AICH  => 0x100;
 use constant RHASH_WHIRLPOOL => 0x200;
 use constant RHASH_RIPEMD160 => 0x400;
-use constant RHASH_GOST      => 0x800;
-use constant RHASH_GOST_CRYPTOPRO => 0x1000;
+use constant RHASH_GOST94    => 0x800;
+use constant RHASH_GOST94_CRYPTOPRO => 0x1000;
 use constant RHASH_HAS160    => 0x2000;
-use constant RHASH_SNEFRU128 => 0x4000;
-use constant RHASH_SNEFRU256 => 0x8000;
+use constant RHASH_GOST12_256 => 0x4000;
+use constant RHASH_GOST12_512 => 0x8000;
 use constant RHASH_SHA224    => 0x10000;
 use constant RHASH_SHA256    => 0x20000;
 use constant RHASH_SHA384    => 0x40000;
@@ -55,16 +57,26 @@ use constant RHASH_SHA3_256  => 0x0800000;
 use constant RHASH_SHA3_384  => 0x1000000;
 use constant RHASH_SHA3_512  => 0x2000000;
 use constant RHASH_CRC32C    => 0x4000000;
-use constant RHASH_ALL       => 0x7FFFFFF;
+use constant RHASH_SNEFRU128 => 0x8000000;
+use constant RHASH_SNEFRU256 => 0x10000000;
+use constant RHASH_ALL       => 0x1FFFFFFF;
 
 ##############################################################################
 # Rhash class methods
 
 # Rhash object constructor
-sub new
+sub new($$@)
 {
-	my $hash_id = $_[1] or die "hash_id not specified";
-	my $context = rhash_init(scalar($hash_id)) or return undef;
+	my $hash_id = 0;
+	shift;
+	scalar(@_) > 0 or die "hash_id not specified";
+	for my $id (@_) {
+		$hash_id |= scalar($id);
+		if(!scalar($id) || (scalar($id) & RHASH_ALL) != $id) {
+			die "bad hash_id = " . scalar($id);
+		}
+	}
+	my $context = rhash_init($hash_id) or return undef;
 	my $self = {
 		context => $context,
 	};
@@ -239,19 +251,21 @@ Crypt::Rhash - Compute hash sums and magnet links
 =head1 SYNOPSIS
 
   use Crypt::Rhash;
-  
   my $msg = "a message text";
   print "MD5 = " . Crypt::Rhash::msg(RHASH_MD5, $msg) . "\n";
-  
+
+  use Crypt::Rhash;
+  my $filepath = "/tmp/file.txt";
   # Calculate two hash functions simultaneously
-  my $r = Crypt::Rhash->new(RHASH_MD5 | RHASH_SHA1);
-  $r->update("a message text")->update(" another message");
-  print  "MD5  = " . $r->hash(RHASH_MD5) . "\n";
-  print  "SHA1 = " . $r->hash(RHASH_SHA1) . "\n";
+  my $r = Crypt::Rhash->new(RHASH_SHA1, RHASH_SHA512);
+  $res = $r->update_file($filepath);
+  defined($res) or die "failed to read $filepath: $!";
+  print "SHA1   = " . $r->hash(RHASH_SHA1) . "\n";
+  print "SHA512 = " . $r->hash(RHASH_SHA512) . "\n";
 
 =head1 DESCRIPTION
 
-Crypt::Rhash module is an object-oriented interface to the LibRHash library,
+Crypt::Rhash module is an object-oriented interface to LibRHash library,
 allowing simultaneous calculation of several hash functions for a file  or a
 text message.
 
@@ -261,18 +275,18 @@ binary format or as a magnet link.
 =head1 SUPPORTED ALGORITHMS
 
 The module supports the following hashing algorithms:
-CRC32,  MD4, MD5,  SHA1, SHA256, SHA512, SHA3,
-AICH, ED2K, Tiger,  DC++ TTH,  BitTorrent BTIH, GOST R 34.11-94, RIPEMD-160,
+CRC32, CRC32C, MD4, MD5, SHA1, SHA256, SHA512, SHA3, AICH, ED2K, Tiger,
+DC++ TTH, GOST R 34.11-94, GOST R 34.11-2012, BitTorrent BTIH, RIPEMD-160,
 HAS-160, EDON-R 256/512, Whirlpool and Snefru-128/256.
 
 =head1 CONSTRUCTOR
 
 Creates and returns new Crypt::Rhash object.
 
-  my $r = Crypt::Rhash->new($hash_id);
-  my $p = new Crypt::Rhash($hash_id); # alternative way to call the constructor
+  my $r = Crypt::Rhash->new($hash_id1, $hash_id2, ...);
+  my $p = new Crypt::Rhash($hash_id1, ...); # alternative way to call the constructor
 
-The $hash_id parameter can be union (via bitwise OR) of any of the following bit-flags:
+Constructor accepts the following constants as arguments:
 
   RHASH_CRC32,
   RHASH_CRC32C,
@@ -286,11 +300,11 @@ The $hash_id parameter can be union (via bitwise OR) of any of the following bit
   RHASH_AICH,
   RHASH_WHIRLPOOL,
   RHASH_RIPEMD160,
-  RHASH_GOST,
-  RHASH_GOST_CRYPTOPRO,
+  RHASH_GOST94,
+  RHASH_GOST94_CRYPTOPRO,
+  RHASH_GOST12_256,
+  RHASH_GOST12_512
   RHASH_HAS160,
-  RHASH_SNEFRU128,
-  RHASH_SNEFRU256,
   RHASH_SHA224,
   RHASH_SHA256,
   RHASH_SHA384,
@@ -300,10 +314,13 @@ The $hash_id parameter can be union (via bitwise OR) of any of the following bit
   RHASH_SHA3_384,
   RHASH_SHA3_512,
   RHASH_EDONR256,
-  RHASH_EDONR512
+  RHASH_EDONR512,
+  RHASH_SNEFRU128,
+  RHASH_SNEFRU256,
+  RHASH_ALL
 
-Also the RHASH_ALL bit mask is the union of all listed bit-flags.
-So the object created via Crypt::Rhash->new(RHASH_ALL) calculates all
+The RHASH_ALL bit mask is bitwise union of all listed above bit-flags.
+An object created as Crypt::Rhash->new(RHASH_ALL) calculates all
 supported hash functions for the same data.
 
 =head1 COMPUTING HASHES
@@ -324,10 +341,11 @@ It returns the $rhash object itself allowing the following construction:
 
 Calculate a hash of the file (or its part) specified by $file_path or a file descriptor $fd.
 The update_fd method doesn't close the $fd, leaving the file position after the hashed block.
-The optional $start and $size specify the block of the file to hash.
+The optional $start and $size specify the block of the file to hash. If $start is undefined,
+then the size is read from the start. If $size is undefined, then the file is read till the end.
 No error is reported if the $size is greater than the number of the unread bytes left in the file.
 
-Returns the number of characters actually read, 0 at end of file, 
+Returns the number of characters actually read, 0 at end of file,
 or undef if there was an error (in the latter case $! is also set).
 
   use Crypt::Rhash;
@@ -343,7 +361,7 @@ or undef if there was an error (in the latter case $! is also set).
 =item $rhash->final()
 
 Finishes calculation for all data buffered by updating methods and stops hash
-calculation. The function is called automatically by any of the 
+calculation. The function is called automatically by any of the
 $rhash->hash*() methods if the final() call was skipped.
 
 =item $rhash->reset()
@@ -451,15 +469,19 @@ Computes and returns a single hash (in its default format) of the $message by th
 
 =head1 LICENSE
 
- Permission is hereby granted, free of charge,  to any person obtaining a copy
- of this software and associated documentation files (the "Software"), to deal
- in the Software without restriction,  including without limitation the rights
- to  use,  copy,  modify,  merge, publish, distribute, sublicense, and/or sell
- copies  of  the Software,  and  to permit  persons  to whom  the Software  is
- furnished to do so.
+                           BSD Zero Clause License
 
- The Software  is distributed in the hope that it will be useful,  but WITHOUT
- ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
- FOR A PARTICULAR PURPOSE.  Use  this  program  at  your  own  risk!
+Copyright (c) 2011, Aleksey Kravchenko
+
+Permission to use, copy, modify, and/or distribute this software for any
+purpose with or without fee is hereby granted.
+
+THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
+REGARD TO THIS SOFTWARE  INCLUDING ALL IMPLIED WARRANTIES OF  MERCHANTABILITY
+AND FITNESS.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
+INDIRECT,  OR CONSEQUENTIAL DAMAGES  OR ANY DAMAGES WHATSOEVER RESULTING FROM
+LOSS OF USE,  DATA OR PROFITS,  WHETHER IN AN ACTION OF CONTRACT,  NEGLIGENCE
+OR OTHER TORTIOUS ACTION,  ARISING OUT OF  OR IN CONNECTION  WITH THE USE  OR
+PERFORMANCE OF THIS SOFTWARE.
 
 =cut
